#!/usr/bin/env python

"""Tests for `nama_alamat` package."""

import pytest


@pytest.fixture
def response():
    """Sample pytest fixture.

    See more at: http://doc.pytest.org/en/latest/fixture.html
    """
    # import requests
    # return requests.get('https://github.com/audreyr/cookiecutter-pypackage')

    from nama_alamat.preprocessing import Preprocessing

    preprocessing_nama = Preprocessing(tipe='nama')
    preprocessing_alamat = Preprocessing(tipe='alamat')

    strings1 = 'IR SULAEMAN'
    strings2 = 'JALAN GARUDA IX BLOK C/3 RT 003 RW 002, JAKARTA UTARA, DKI JAKARTA 12110'
    strings3 = 'A L I'
    strings4 = 'DONI DONI'

    return (
        preprocessing_nama.preprocessing(strings1)
        + preprocessing_alamat.preprocessing(strings2)
        + preprocessing_nama.preprocessing(strings3)
        + preprocessing_nama.preprocessing(strings4)
    )


def test_content(response):
    """Sample pytest test function with the pytest fixture as an argument."""
    # from bs4 import BeautifulSoup
    # assert 'GitHub' in BeautifulSoup(response.content).title.string
    assert 'sulaeman' in response
    assert 'jakarta' in response
    del response



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['nama_alamat',
 'nama_alamat.preprocessing',
 'nama_alamat.preprocessing.dict_files',
 'tests']

package_data = \
{'': ['*']}

install_requires = \
['roman>=3.3,<4.0']

extras_require = \
{'dev': ['tox>=3.20.1,<4.0.0',
         'virtualenv>=20.2.2,<21.0.0',
         'pip>=20.3.1,<21.0.0',
         'twine>=3.3.0,<4.0.0',
         'pre-commit>=2.12.0,<3.0.0',
         'toml>=0.10.2,<0.11.0',
         'bump2version>=1.0.1,<2.0.0'],
 'doc': ['mkdocs>=1.1.2,<2.0.0',
         'mkdocs-include-markdown-plugin>=1.0.0,<2.0.0',
         'mkdocs-material>=6.1.7,<7.0.0',
         'mkdocstrings>=0.15.2,<0.16.0',
         'mkdocs-autorefs>=0.2.1,<0.3.0'],
 'test': ['black>=21.5b2,<22.0',
          'isort>=5.8.0,<6.0.0',
          'flake8>=3.9.2,<4.0.0',
          'flake8-docstrings>=1.6.0,<2.0.0',
          'mypy>=0.900,<0.901',
          'pytest>=6.2.4,<7.0.0',
          'pytest-cov>=2.12.0,<3.0.0']}

setup_kwargs = {
    'name': 'nama-alamat',
    'version': '0.2.7',
    'description': 'Indonesia Name and Address Preprocessor.',
    'long_description': "# Indonesia Name and Address Preprocessor\n\n[![pypi](https://img.shields.io/pypi/v/nama_alamat.svg)](https://pypi.org/project/nama_alamat/)\n[![python](https://img.shields.io/pypi/pyversions/nama_alamat.svg)](https://pypi.org/project/nama_alamat/)\n[![Build Status](https://github.com/kloworizer/nama_alamat/actions/workflows/dev.yml/badge.svg)](https://github.com/kloworizer/nama_alamat/actions/workflows/dev.yml)\n[![codecov](https://codecov.io/gh/kloworizer/nama_alamat/branch/main/graphs/badge.svg)](https://codecov.io/github/kloworizer/nama_alamat)\n\nIndonesia Name and Address Preprocessor\n\n-   Documentation: <https://kloworizer.github.io/nama_alamat>\n-   GitHub: <https://github.com/kloworizer/nama_alamat>\n-   PyPI: <https://pypi.org/project/nama_alamat/>\n-   Free software: MIT\n\n## Features\n\n-   Preprocessing Indonesia name and address.\n\n## Instalation\n\n```\npip install nama-alamat\n```\n\n## Usage\n\nexample of name preproccesing:\n```\nfrom nama_alamat.preprocessing import Preprocessing\npreprocessing_nama = Preprocessing(tipe='nama')\nstrings = 'IR SULAEMAN'\nprint(preprocessing_nama.preprocessing(strings))\n```\n\naddress preproccesing:\n```\nfrom nama_alamat.preprocessing import Preprocessing\npreprocessing_alamat = Preprocessing(tipe='alamat')\nstrings = 'JALAN GARUDA IX BLOK C/3 RT 003 RW 002, JAKARTA UTARA, DKI JAKARTA 12110'\nprint(preprocessing_alamat.preprocessing(strings))\n```\n\n## Credits\n\nThis package was created with [Cookiecutter](https://github.com/audreyr/cookiecutter) and the [waynerv/cookiecutter-pypackage](https://github.com/waynerv/cookiecutter-pypackage) project template.\n",
    'author': 'Esha Indra',
    'author_email': 'esha.indra@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/kloworizer/nama_alamat',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=3.6.2,<4.0',
}


setup(**setup_kwargs)



"""Module for preprocessing Indonesia Name and Address.

Dictionary for preproccesing on dict_files folder.
Roman library used for converting roman number to arabic.
"""

# import library
import os
import re
from functools import reduce

import roman

# membuat dictionary untuk preprocessing nama dan alamat
dict_alamat = {}
dict_nama = {}

here = os.path.dirname(os.path.abspath(__file__))

# menambahkan tiap baris dari file txt ke dictionary
with open(os.path.join(here, 'dict_files', 'dict_alamat.txt'), 'r') as file:
    for line in file:
        key, value = line.replace('\'', '').rstrip('\n').split(':')
        dict_alamat[key] = value

with open(os.path.join(here, 'dict_files', 'dict_nama.txt'), 'r') as file:
    for line in file:
        key, value = line.replace('\'', '').rstrip('\n').split(':')
        dict_nama[key] = value


class Preprocessing:
    """Preprocessing class."""

    def __init__(self, tipe='alamat'):
        """Class initialization.

        Args:
            tipe (str, optional): valid value : 'alamat' for address or 'nama' for name. Defaults to 'alamat'.
        """
        self.tipe = tipe

    # standarisasi penulisan nama dan alamat
    def standardize(self, strings):
        """Standardize function.

        Args:
            strings (_type_): input string

        Returns:
            _type_: returning standarized string based on type (name or address)
        """
        tipe = self.tipe
        if tipe == 'alamat':
            result = " ".join(dict_alamat.get(ele, ele) for ele in strings.split())
        else:
            result = re.sub(r'\s', '_', strings)
            for i, k in dict_nama.items():
                str_from = '(_|^)' + i + '(_|$)'
                str_to = '_' + k + '_'
                result = re.sub(str_from, str_to, result)
            result = re.sub('_', ' ', result)
        return result

    def preprocessing(self, strings):
        """Preprocessing function.

        Args:
            strings (_type_): input string

        Returns:
            _type_: return preprocessed string
        """
        tipe = self.tipe

        # kata-kata tidak berguna
        stopword = [
            'please specify',
            'hold mail',
            'holdmail',
            'dummy',
            'unknown',
            'middlename',
            'npwp',
            'qq',
            'sp_xplor',
            'null',
            'anonymous',
            'not_associate',
        ]

        if isinstance(strings, str):
            # lowercase
            result = strings.lower()

            # remove non ascii chars
            result = re.sub(r'[^\x00-\x7f]', '', result)

            # remove old style name
            if re.match(r'^(?:\w ){2,}[A-z]($|\W)', result):
                result = ''.join(result.split())

            # remove inside bracket
            result = re.sub(r'\([^)]*\)', '', result)

            # remove stopword
            result = reduce(lambda a, b: a.replace(b, ''), stopword, result)

            if tipe == 'nama':
                # remove number
                result = re.sub(r'\d+', '', result)
            if tipe == 'alamat':
                # remove kodepos
                result = re.sub(r' \d\d\d\d\d', '', result)

            # remove punctuation
            result = re.sub(r'[^\w\s]', ' ', result)

            # remove whitespace
            result = result.strip()

            # remove double space
            result = re.sub(r'\s+', ' ', result)

            # standardize
            result = self.standardize(result)

            # remove whitespace
            result = result.strip()

            # remove double space
            result = re.sub(r'\s+', ' ', result)

            # hapus nama 1 kata diulang
            if tipe == 'nama':
                result = [x.strip() for x in result.split()]
                result = ' '.join(list(dict.fromkeys(result)))

            # roman to arabic
            if tipe == 'alamat':
                result = ' '.join(
                    [
                        str(roman.fromRoman(x))
                        if re.match("(^(?=[MDCLXVI])M*(C[MD]|D?C{0,3})(X[CL]|L?X{0,3})(I[XV]|V?I{0,3})$)", x)
                        else x
                        for x in result.upper().split()
                    ]
                ).lower()

            return result
        else:
            return strings
